//*****************************************************************************************
// Truevision - a 3d modeler for gnome and povray
//
// cone.cc
//
// Vincent LE PRINCE <vincentleprince@users.sourceforge.net>
// Copyright (C) 2000-2001 Vincent LE PRINCE
// This file is part of the TRUEVISION Package

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */ 
//*******************************************************************************************
#include "cone.h"
#include "viewmanager.h"
#include "objectlist.h"
#include "tvio.h"
#include <GL/glu.h>

//**************************************
// Constructeur
//**************************************
Cone::Cone( app_objs *appref ) : Object3D_with_material( appref )
{
type = TV_OBJ3D_CONE;
category = TV_OBJ3D_OBJECTS;
set_name( "Cone" );

// Base
location = new ObjParam_point( N_("Location"), "LOC", NULL, app_ref, true );
location->set( 0, 0, 0 );
base = new ObjParam_float( N_("Base radius"), "BASE", NULL, app_ref, true, 2 );
base->set_range( 1000, -1000, 0.1, 5 );
cap = new ObjParam_float( N_("Cap radius"), "CAP", NULL, app_ref, true, 1 );
cap->set_range( 1000, -1000, 0.1, 5 );
size = new ObjParam_scale( N_("Size"), "SIZE", NULL, app_ref, true );
size->set( 0.2, 0.2, 0.2 );
rotation = new ObjParam_rotation( N_("Rotation"), "ROT", NULL, app_ref, true );
rotation->set( 0, 0, 0 );
open = new ObjParam_bool ( N_("Open"), "OPEN", NULL, app_ref, true, false );
}

Cone::Cone( Cone & ref ) : Object3D_with_material( ref )
{
location = new ObjParam_point( *ref.location );
size = new ObjParam_scale( *ref.size );
rotation = new ObjParam_rotation( *ref.rotation );
cap = new ObjParam_float( *ref.cap );
base = new ObjParam_float( *ref.base );
open = new ObjParam_bool( *ref.open );
}

Cone::~Cone()
{
delete location;
delete size;
delete rotation;
delete cap;
delete base;
delete open;
}

//**************************************
// Display
// Dessin de la boite
//**************************************
void Cone::display( glview *view )
{
if ( hidden->value() ) return;
if ( location->changed() || size->changed() || rotation->changed() || base->changed()  || cap->changed() || open->changed() )
	list.invalidate();

Object3D::display( view );		
if ( ! list.exec() )
	{
	// mise  zero des parametres
	location->unchange();
	size->unchange();
	rotation->unchange();
	base->unchange();
	cap->unchange();
	open->unchange();

	// creation de la liste si necessaire
	list.begin();
	glPushMatrix();
	set_color();

	// Position et direction
	gfloat x, y, z;
	location->get( x, y, z );
	glTranslatef( x, y, z );
	rotation->gl_rotate();	
	size->get( x, y, z );
	glScalef( x, y, z );
	glTranslatef( 0, 0, -1 );

	GLUquadricObj *quadobj = gluNewQuadric();
	if ( !quadobj ) { app_warning( N_("Out of memory" ) ); return; }	
	gluQuadricDrawStyle( quadobj, GLU_FILL );
	gluQuadricNormals( quadobj, GLU_SMOOTH );
	gluQuadricOrientation( quadobj, GLU_OUTSIDE );
    gluCylinder(quadobj, base->value(), cap->value(), 2, 12, 12 );

	if ( ! open->value() )
		{
		GLUquadricObj *disk1 = gluNewQuadric();
		if ( !quadobj ) { app_warning( N_("Out of memory" ) ); return; }	
		gluQuadricDrawStyle( disk1, GLU_FILL );
		gluQuadricNormals( disk1, GLU_SMOOTH );
		gluQuadricOrientation( disk1, GLU_OUTSIDE );
	    gluDisk( disk1, 0, base->value(), 12, 6 );

	   	glTranslatef( 0, 0, 2 );
		GLUquadricObj *disk2 = gluNewQuadric();
		if ( !quadobj ) { app_warning( N_("Out of memory" ) ); return; }	
		gluQuadricDrawStyle( disk2, GLU_FILL );
		gluQuadricNormals( disk2, GLU_SMOOTH );
		gluQuadricOrientation( disk2, GLU_OUTSIDE );
	    gluDisk( disk2, 0, cap->value(), 12, 6 );    	
	    }
	
	glEnd();
	glPopMatrix();
	list.end();
	}
}


//***********************************************
// Edit
//***********************************************
void Cone::edit_widget( GtkWidget *wid )
{
bool tt = true;
// Options communes
Object3D::edit_widget( wid );

// Options de geometrie
new_frame( edit_cont, _("General settings") );
base->get_widget( frame, tt );
cap->get_widget( frame, tt );
location->get_widget( frame, tt );
size->get_widget( frame, tt );
rotation->get_widget( frame, tt );
open->get_widget( frame, tt );
texture->get_widget( edit_cont, tt );

gtk_widget_show_all( wid );
}

//***********************************************
// Mouse drag
//***********************************************
void Cone::mouse_drag( struct drag_info *drag )
{
VMAN_DEF
OBJLIST_DEF

switch( vmanager->get_pointer_mode() )
	{
	case TV_PMODE_TRANSLATE:
		location->mouse_drag( drag );
		break;

	case TV_PMODE_SCALE:
		{ size->mouse_drag( drag ); }
		break;

	case TV_PMODE_ROTATE:
		{ rotation->mouse_drag( drag ); break; }

	case TV_PMODE_CUSTOM:
		((ObjParam_point*)(objlist->get_current_param()))->mouse_drag( drag );
		break;

	default: 
		break;
	}
}

//***********************************************
// Pref_changed
//***********************************************
void Cone::pref_changed()
{
Object3D::pref_changed();
location->pref_changed();
size->pref_changed();
base->pref_changed();
cap->pref_changed();
rotation->pref_changed();
open->pref_changed();
}

//***********************************************
// Destroy editor
//***********************************************
void Cone::destroy_editor()
{
Object3D::destroy_editor();
location->clear_widget();
size->clear_widget();
texture->clear_widget();
rotation->clear_widget();
base->clear_widget();
cap->clear_widget();
open->clear_widget();
}

//***********************************************
// Output to povray
//***********************************************
void Cone::output_to_povray( ofstream & file )
{
if ( !render->value() ) return;
file << "\n\n//Cone " << name->value();
file << "\ncone {\n\t<0,0,1>, " << base->value();
file << ", <0, 0, -1>, " << cap->value() << "\n\t";
if ( open->value() ) file << "open\n\t";
texture->output_to_povray( file );

float x, y, z;
size->get( x, y, z );
file << "\n\tscale <" << x << ',' << y << ',' << z << ">\n\t";
rotation->output_to_povray( file );
location->get( x, y, z );
file << "\n\ttranslate <" << x << "," << y << "," << -z << ">";
Object3D::output_to_povray( file );
file << "\n}";
}

void Cone::save( ofstream & file )
{
file << "\nCONE{\n";
save_basics( file );
base->save( file );
cap->save( file );
size->save( file );
rotation->save( file );
location->save( file );
open->save( file );
texture->save( file );
file << "\n}";
}

bool Cone::load( ifstream & file, char *ltag )
{
if ( strcmp( ltag, "CONE" ) ) return false;
set_load_progress( file );

char * tag = NULL;
do {
	tag = tvio_get_next_tag( file );
	if ( tag == NULL ) break;
	
	if ( load_basics( file, tag ) ) continue;
 	if ( location->load( file, tag ) ) continue;
 	if ( size->load( file, tag ) ) continue;
 	if ( rotation->load( file, tag ) ) continue;
 	if ( base->load( file, tag ) ) continue;
 	if ( cap->load( file, tag ) ) continue;
 	if ( open->load( file, tag ) ) continue;
		
	tvio_skip_section(file );
	} while ( tag != NULL );

return true;
}
