//*****************************************************************************************
// Truevision - a 3d modeler for gnome and povray
//
// preferences.cc
//
// Vincent LE PRINCE <vincentleprince@users.sourceforge.net>
// Copyright (C) 2000-2001 Vincent LE PRINCE
// This file is part of the TRUEVISION Package

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */ 
//*******************************************************************************************
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <iostream.h>
#include "preferences.h"
#include "interface.h"
#include "dlgutils.h"
#include "tvio.h"


//*********************************************
// Preferences
// Constructeur
//*********************************************
Preferences::Preferences( app_objs * appr )
{
// init
app_ref = appr;
app_ref->prefer = (app_object*)this;
changed = false;
dialog = NULL;
selected_page = 0;

// Geometry
save_geo = new TvWidget_bool( N_("Save geometry"), "SAVEGEO", N_("Save window position and size and restore it the next session"), app_ref, true );
PrefList.push_back( save_geo );
win_xpos = new TvWidget_int( NULL, "WINXPOS", NULL, app_ref, 0 );
PrefList.push_back( win_xpos );
win_ypos = new TvWidget_int( NULL, "WINYPOS", NULL, app_ref, 0 );
PrefList.push_back( win_ypos );
win_xsize = new TvWidget_int( NULL, "WINXSIZE", NULL, app_ref, 800 );
PrefList.push_back( win_xsize );
win_ysize = new TvWidget_int( NULL, "WINYSIZE", NULL, app_ref, 600 );
PrefList.push_back( win_ysize );

tbar_selected  = new TvWidget_color( N_("Focused view bar color"), "TBSCOL", NULL, app_ref );
tbar_selected->set( 0.32, 0.45, 0.69 );
PrefList.push_back( tbar_selected );
tbar_unselected  = new TvWidget_color( N_("Unfocused view bar color"), "TBUSCOL", NULL, app_ref );
tbar_unselected->set( 0.82, 0.82, 0.82 );
PrefList.push_back( tbar_unselected );


// Interface options
tooltips = new TvWidget_bool( N_("Show tooltips"), "TOOLTIPS", _("Show ints on interface controls"), app_ref, true );
PrefList.push_back( tooltips );
toolbar_text = new TvWidget_bool( N_("Show labels in toolbar"), "TBTEXT", _("Display text & icons in toolbar"), app_ref, false );
PrefList.push_back( toolbar_text );
toolbar = new TvWidget_bool( N_("Show toolbar"), "TOOLB", _("Show toolbar"), app_ref, true );
PrefList.push_back( toolbar );
matedit = new TvWidget_bool( N_("Open editor at material creation"), "MATED", NULL, app_ref, false );
PrefList.push_back( matedit );
undo_limit = new TvWidget_int( N_("Maximum undo level"), "UNDOL", NULL, app_ref, 10 );
PrefList.push_back( undo_limit );

// Axis & axis labels
axis2D = new TvWidget_bool( N_("Show axis in 2d view"), "AXIS2D", NULL, app_ref, true );
PrefList.push_back( axis2D );
axis3D = new TvWidget_bool( N_("Show axis in 3d view"), "AXIS3D", NULL, app_ref, true );
PrefList.push_back( axis3D );
axis_color = new TvWidget_color( N_("Axis color"), "AXISCOL", NULL, app_ref );
axis_color->set( 0.44, 0.44, 0.44 );
PrefList.push_back( axis_color );
labels2D = new TvWidget_bool( N_("Show axis labels in 2d view"), "LAB2D", NULL, app_ref, true );
PrefList.push_back( labels2D );
labels3D = new TvWidget_bool( N_("Show axis labels in 3d view"), "LAB3D", NULL, app_ref, true );
PrefList.push_back( labels3D );
labels_color = new TvWidget_color( N_("Labels color"), "LABCOL", NULL, app_ref );
labels_color->set( 0.44, 0.44, 0.44 );
PrefList.push_back( labels_color );

// Grids
grid2D = new TvWidget_bool( N_("Show grid in 2d view"), "2DGRID", NULL, app_ref, true );
PrefList.push_back( grid2D );
gridxy = new TvWidget_bool( N_("Show XY grid in 2d view"), "XYGRID", NULL, app_ref, false );
PrefList.push_back( gridxy );
gridxz = new TvWidget_bool( N_("Show XZ grid in 2d view"), "XZGRID", NULL, app_ref, true );
PrefList.push_back( gridxz );
gridyz = new TvWidget_bool( N_("Show YZ grid in 2d view"), "YZGRID", NULL, app_ref, false );
PrefList.push_back( gridyz );
gridxy_color = new TvWidget_color( N_("XY grid color"), "XYGCOL", NULL, app_ref );
gridxy_color->set( 0.52, 0.52, 0.52 );
PrefList.push_back( gridxy_color );
gridxz_color = new TvWidget_color( N_("XZ grid color"), "XZGCOL", NULL, app_ref );
gridxz_color->set( 0.52, 0.52, 0.52 );
PrefList.push_back( gridxz_color );
gridyz_color = new TvWidget_color( N_("YZ grid color"), "XYGCOL", NULL, app_ref );
gridyz_color->set( 0.52, 0.52, 0.52 );
PrefList.push_back( gridyz_color );

// Default view modes
view_modes[0] = new TvWidget_int( NULL, "VIEW0MOD", NULL, app_ref, 0 );
PrefList.push_back( view_modes[0] );
view_modes[1] = new TvWidget_int( NULL, "VIEW1MOD", NULL, app_ref, 1 );
PrefList.push_back( view_modes[1] );
view_modes[2] = new TvWidget_int( NULL, "VIEW2MOD", NULL, app_ref, 2 );
PrefList.push_back( view_modes[2] );
view_modes[3] = new TvWidget_int( NULL, "VIEW3MOD", NULL, app_ref, 3 );
PrefList.push_back( view_modes[3] );

// OpenGL options
line_antialias = new TvWidget_bool( N_("Antialiasing"), "ANTIALIAS", _("Smooth lines"), app_ref, false );
PrefList.push_back( line_antialias );
zoom2d = new TvWidget_bool( N_("Link 2d zoom"), "ZOOM2D", _("Use the same zoom factor for every 2d view"), app_ref, true );
PrefList.push_back( zoom2d );
nav2D = new TvWidget_bool( N_("Link view centers"), "NAV2D", _("Use the same center of interest for every view"), app_ref, true );
PrefList.push_back( nav2D );

// Colors
v3dbkg_color = new TvWidget_color( N_("3d view background color"), "3DBKG", NULL, app_ref );
v3dbkg_color->set( 0.7, 0.7, 0.7 );
PrefList.push_back( v3dbkg_color );
v2dbkg_color = new TvWidget_color( N_("2d view background color"), "2DBKG", NULL, app_ref );
v2dbkg_color->set( 0.7, 0.7, 0.7 );
PrefList.push_back( v2dbkg_color );

obj_color = new TvWidget_color( N_("Unselected object color"), "OBJCOL", NULL, app_ref );
obj_color->set( 0.94, 0.94, 0.94 );
PrefList.push_back( obj_color );
objsel_color = new TvWidget_color( N_("Selected object color"), "OBJSCOL", NULL, app_ref );
objsel_color->set( 0.14, 0.14, 1.00 );
PrefList.push_back( objsel_color );

// Paths
povcmd = new TvWidget_path( N_("x-povray command"), "POVCMD", _("Command used to launch xpovray or megapov"), app_ref );
povcmd->set( "x-povray" );
PrefList.push_back( povcmd );
author = new TvWidget_entry( N_("Default author name"), "AUTH", N_("Name set by default in author entries"), app_ref );
PrefList.push_back( author );

// Home
const char *home_dir = "/.truevision";
char *home_base = getenv( "HOME" );
home = new char[ strlen(home_base) + strlen( home_dir ) + 1 ];
strcpy( home, home_base );
strcat( home, home_dir );
if ( access( home, F_OK ) == -1 ) mkdir( home, S_IRWXU );


// Preferences file
const char * pref_file_name = "/preferences";
pref_file = new char[ strlen(home) + strlen( pref_file_name ) + 1 ];
strcpy( pref_file, home );
strcat( pref_file, pref_file_name );
load();
}

//***********************************************
// Load
//***********************************************
void Preferences::load()
{
if ( access( pref_file, F_OK ) == -1 ) return;

ifstream file( pref_file, ios::binary );
char *tag = NULL;
do
	{
	tag = tvio_get_next_tag( file );
	if ( tag == NULL ) break;

	bool found = false;
	for ( register unsigned int i = 0 ; i < PrefList.size() ; i++ )
		if ( PrefList[i]->load( file , tag ) ) { found = true; break; }
	if ( found ) continue;

	tvio_skip_section( file );
	}
while( tag != NULL );
return;

file.close();
}


//***********************************************
// Save
//***********************************************
void Preferences::save()
{
// rcup des donnes
((Interface*)(app_ref->interf))->flush_pref();

ofstream file( pref_file, ios::binary );
if ( file == NULL ) { app_warning( _("Cannot create preferences file !" ) ); }
for ( register unsigned int i = 0 ; i < PrefList.size() ; i ++ )
	{
	PrefList[i]->flush();
	PrefList[i]->save( file );
	file << '\n';
	}
file.close();	
}


//**********************************************
// Construction du dialogue
//**********************************************
void Preferences::raise_dlg()
{
if ( GNOME_IS_DIALOG(dialog) )
	{
	gdk_window_raise( dialog->window );
	return;
	}
applied = false;

// Sauvegarde pour annulation
for ( register unsigned int i = 0 ; i < PrefList.size() ; i++ )
	PrefList[i]->store();
	
dialog = gnome_dialog_new( N_("Preferences"), GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_APPLY, GNOME_STOCK_BUTTON_CANCEL, NULL );
gtk_signal_connect( GTK_OBJECT(dialog), "clicked", GTK_SIGNAL_FUNC(dlg_click), this );
gtk_signal_connect( GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(dlg_destroy), this );

// Pager
notebk = gtk_notebook_new();
gtk_notebook_set_scrollable( GTK_NOTEBOOK(notebk), TRUE );
gtk_box_pack_start( GTK_BOX(GNOME_DIALOG(dialog)->vbox), notebk, FALSE, TRUE, 5 );
GtkWidget *vbox;
GtkWidget *box1, *box2;
bool tt = tooltips->value();

// Page Window
vbox = dlg_new_page( notebk, _("Window") );
dlg_double_box_frame( _("Interface"), vbox, box1, box2 );
	save_geo->get_widget( box1, tt );
	tooltips->get_widget( box1, tt );
	toolbar->get_widget( box2, tt );	
	toolbar_text->get_widget( box2, tt );
	tbar_selected->get_widget( box1, tt );
	tbar_unselected->get_widget( box1, tt );
    matedit->get_widget( box2, tt );
    undo_limit->get_widget( box2, tt );

// Page View
vbox = dlg_new_page( notebk, _("View") );
dlg_double_box_frame( _("Axis"), vbox, box1, box2 );
	axis2D->get_widget( box1, tt );
	axis3D->get_widget( box1, tt );
	labels3D->get_widget( box1, tt );
	labels2D->get_widget( box1, tt );	
	axis_color->get_widget( box2, tt );
	labels_color->get_widget( box2, tt );

dlg_double_box_frame( _("Grid"), vbox, box1, box2 );
	gridxy->get_widget( box1, tt );
	gridxz->get_widget( box1, tt );
	gridyz->get_widget( box1, tt );
	grid2D->get_widget( box1, tt );
	gridxy_color->get_widget( box2, tt );
	gridxz_color->get_widget( box2, tt );
	gridyz_color->get_widget( box2, tt );

dlg_double_box_frame( _("Misc."), vbox, box1, box2 );
	v2dbkg_color->get_widget( box1, tt );
	v3dbkg_color->get_widget( box1, tt );
	obj_color->get_widget( box1, tt );
	objsel_color->get_widget( box1, tt );
	line_antialias->get_widget( box2, tt );
	zoom2d->get_widget( box2, tt );
	nav2D->get_widget( box2, tt );

// Page Povray
vbox = dlg_new_page( notebk, _("Path") );
dlg_double_box_frame( _("Options"), vbox, box1, box2 );
	povcmd->get_widget( box1, tt );
	author->get_widget( box1, tt );

gtk_widget_show_all(dialog);
gtk_notebook_set_page( GTK_NOTEBOOK(notebk), selected_page );
}


// Rsultat
void Preferences::clicked( gint button )
{
switch( button )
	{
	case 0:
		flush_dlg();
		close_dlg();
		send_message();
		break;

	case 1:
		flush_dlg();
		send_message();
		applied = true;
		break;

	default:
	case 2:
		if ( applied ) { undo_dlg(); send_message(); }
		close_dlg();
		break;
	}
}

//***********************************************************
// Message
//***********************************************************
void Preferences::send_message()
{
app_ref->interf->pref_changed();
app_ref->toolbar->pref_changed();
app_ref->obj3dlist->pref_changed();
app_ref->vmanager->pref_changed();
app_ref->scene->pref_changed();
app_ref->texlist->pref_changed();
pref_changed();  // Me, too ;-)
}

void Preferences::close_dlg()
{
selected_page = gtk_notebook_get_current_page( GTK_NOTEBOOK(notebk) );
if ( GNOME_IS_DIALOG(dialog) ) gnome_dialog_close( GNOME_DIALOG(dialog) );
dialog = NULL;
for ( register unsigned int i = 0 ; i < PrefList.size()  ; i++ )
	PrefList[i]->clear_widget();
}
